@@ -43,6 +43,8 @@ enum ReaderWriterInternal<T> {
     Fs(FsClient),
 }
 
+type Size = u64;
+
 impl<T> ReaderWriterInternal<T>
 where
     T: crate::oauth2::token::TokenGenerator,
@@ -77,7 +79,13 @@ where
         }
     }
 
-    async fn write<S>(&self, path: &RelativePath, stream: S) -> RSyncResult<()>
+    async fn write<S>(
+        &self,
+        mtime: Option<chrono::DateTime<chrono::Utc>>,
+        set_fs_mtime: bool,
+        path: &RelativePath,
+        stream: S,
+    ) -> RSyncResult<()>
     where
         S: futures::TryStream<Ok = bytes::Bytes, Error = RSyncError>
             + Send
@@ -86,8 +94,14 @@ where
             + 'static,
     {
         match self {
-            ReaderWriterInternal::Gcs(client) => client.write(path, stream).await,
-            ReaderWriterInternal::Fs(client) => client.write(path, stream).await,
+            ReaderWriterInternal::Gcs(client) => match mtime {
+                Some(mtime) => client.write_mtime(mtime, path, stream).await,
+                None => client.write(path, stream).await,
+            },
+            ReaderWriterInternal::Fs(client) => match (mtime, set_fs_mtime) {
+                (Some(mtime), true) => client.write_mtime(mtime, path, stream).await,
+                _ => client.write(path, stream).await,
+            },
         }
     }
 
@@ -104,12 +118,23 @@ where
             ReaderWriterInternal::Fs(client) => client.exists(path).await,
         }
     }
+
+    async fn size_and_mt(
+        &self,
+        path: &RelativePath,
+    ) -> RSyncResult<(Option<chrono::DateTime<chrono::Utc>>, Option<Size>)> {
+        match self {
+            ReaderWriterInternal::Gcs(client) => client.size_and_mt(path).await,
+            ReaderWriterInternal::Fs(client) => client.size_and_mt(path).await,
+        }
+    }
 }
 
 pub type DefaultRSync = RSync<crate::oauth2::token::AuthorizedUserCredentials>;
 pub struct RSync<T> {
     source: ReaderWriterInternal<T>,
     dest: ReaderWriterInternal<T>,
+    restore_fs_mtime: bool,
 }
 
 impl<T> RSync<T>
@@ -120,35 +145,69 @@ where
         Self {
             source: source.inner,
             dest: dest.inner,
+            restore_fs_mtime: false,
         }
     }
 
-    async fn write_entry(&self, path: &RelativePath) -> RSyncResult<()> {
+    pub fn with_restore_fs_mtime(mut self, restore_fs_mtime: bool) -> Self {
+        self.restore_fs_mtime = restore_fs_mtime;
+        self
+    }
+
+    async fn write_entry(
+        &self,
+        mtime: Option<chrono::DateTime<chrono::Utc>>,
+        path: &RelativePath,
+    ) -> RSyncResult<()> {
         let source = self.source.read(path).await;
-        self.dest.write(path, source).await?;
+        self.dest
+            .write(mtime, self.restore_fs_mtime, path, source)
+            .await?;
         Ok(())
     }
 
-    async fn sync_entry(&self, path: &RelativePath) -> RSyncResult<RSyncStatus> {
-        let crc = self.dest.get_crc32c(path).await?;
-
-        Ok(match crc {
+    async fn sync_entry_crc32c(&self, path: &RelativePath) -> RSyncResult<RSyncStatus> {
+        Ok(match self.dest.get_crc32c(path).await? {
             None => {
-                self.write_entry(path).await?;
-                RSyncStatus::Created(path.to_owned())
+                self.write_entry(None, path).await?;
+                RSyncStatus::updated("no dest crc32c", path)
             }
             Some(crc32c_dest) => {
                 let crc32c_source = self.source.get_crc32c(path).await?;
                 if Some(crc32c_dest) == crc32c_source {
-                    RSyncStatus::AlreadySynced(path.to_owned())
+                    RSyncStatus::already_synced("same crc32c", path)
                 } else {
-                    self.write_entry(path).await?;
-                    RSyncStatus::Updated(path.to_owned())
+                    self.write_entry(None, path).await?;
+                    RSyncStatus::updated("different crc32c", path)
                 }
             }
         })
     }
 
+    async fn sync_entry(&self, path: &RelativePath) -> RSyncResult<RSyncStatus> {
+        Ok(match self.dest.size_and_mt(path).await? {
+            (Some(dest_dt), Some(dest_size)) => match self.source.size_and_mt(path).await? {
+                (Some(source_dt), Some(source_size)) => {
+                    let dest_ts = dest_dt.timestamp();
+                    let source_ts = source_dt.timestamp();
+                    if dest_ts == source_ts && dest_size == source_size {
+                        RSyncStatus::already_synced("same mtime and size", path)
+                    } else {
+                        self.write_entry(Some(source_dt), path).await?;
+                        RSyncStatus::updated("different size or mtime", path)
+                    }
+                }
+                _ => self.sync_entry_crc32c(path).await?,
+            },
+            (None, None) => {
+                let (mtime, _) = self.source.size_and_mt(path).await?;
+                self.write_entry(mtime, path).await?;
+                RSyncStatus::Created(path.to_owned())
+            }
+            _ => self.sync_entry_crc32c(path).await?,
+        })
+    }
+
     /// Sync synchronize source to destination by comparing crc32c if destination already exists
     ///
     /// Example
@@ -362,8 +421,22 @@ impl std::error::Error for RSyncError {}
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
 pub enum RSyncStatus {
     Created(RelativePath),
-    Updated(RelativePath),
-    AlreadySynced(RelativePath),
+    Updated { reason: String, path: RelativePath },
+    AlreadySynced { reason: String, path: RelativePath },
+}
+
+impl RSyncStatus {
+    fn updated(reason: &str, path: &RelativePath) -> Self {
+        let reason = reason.to_owned();
+        let path = path.to_owned();
+        Self::Updated { reason, path }
+    }
+
+    fn already_synced(reason: &str, path: &RelativePath) -> Self {
+        let reason = reason.to_owned();
+        let path = path.to_owned();
+        Self::AlreadySynced { reason, path }
+    }
 }
 
 #[derive(Debug, PartialEq, Eq, PartialOrd, Ord)]
